/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.Commons;

import com.google.gwt.event.dom.client.FocusEvent;
import com.google.gwt.event.dom.client.FocusHandler;
import com.google.gwt.event.dom.client.MouseOverEvent;
import com.google.gwt.event.dom.client.MouseOverHandler;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Widget;

import java.util.Date;


/**
 * This abstract class represents a grid with titles and columns, effects on the row, 
 * sort functionalities ...
 * 
 * @author Mathieu Crassin
 * 
 */
public abstract class SpecialGrid extends Grid {

  // the columns' titles of the grid
  protected String []columnTitles = null;
  
  // boolean to set if the sort is active or not (for details on process or instnace in activity view)
  protected boolean activeSort = true;
  
  // the index of the column sorted
  protected int sortColumnIndex = 0;
  
  // the order of the sort
  protected boolean order = false; 
  
  // it's the string to add after a too wide label
  protected String exceededString = "...";
  
  // this string to change the style easily on mouseOver
  private static final String WHICHSTYLE = "MouseOver";
  
  
  /**
   * Empty constructor that only calls the super constructor.
   * 
   * @author Mathieu Crassin
   * 
   */
  public SpecialGrid() {
    //
    super();
  }
  
  
  /**
   * Constructor of the grid.
   * 
   * @author Mathieu Crassin
   * @param columnTitles : the strings table with grid columns titles
   * @param initialSortColumnIndex : the index of the column to sort
   * @param activeSort : boolean to activate the sort or not
   * 
   */  
  public SpecialGrid(final String []columnTitles, final int initialSortColumnIndex, final boolean activeSort) {
    //
    super();
    //
    this.activeSort = activeSort;
    //
    initGrid(columnTitles, initialSortColumnIndex);
  }

  
  /**
   * Function that stores the column titles, initialize the grid, and init the sort function.
   * 
   * @author Mathieu Crassin
   * @param gridColumnTitles : the grid columns titles
   * @param initialSortColumnIndex : the column to sort
   * 
   */  
  public void initGrid(final String []gridColumnTitles, final int initialSortColumnIndex) {
    //
    this.columnTitles = gridColumnTitles;
    //
    manageGrid();
    // initialize the list to sort by state
    this.sortColumnIndex = initialSortColumnIndex;
    this.order = true;    
  }
  
  
  /**
   * Getter of columnIndex parameter
   * 
   * @author Mathieu Crassin
   * @return int : the columnIndex value
   * 
   */  
  public int getColumnIndex() {
    return sortColumnIndex;
  }
  
  
  /**
   * Setter of columnIndex parameter
   * 
   * @author Mathieu Crassin
   * @param columnIndex : the columnIndex value
   * 
   */  
  public void setColumnIndex(int columnIndex) {
    this.sortColumnIndex = columnIndex;
  }
  
  
  /**
   * Getter of order parameter
   * 
   * @author Mathieu Crassin
   * @return boolean : the order value
   * 
   */  
  public boolean getOrder() {
    return order;
  }
  
  
  /**
   * Setter of order parameter
   * 
   * @author Mathieu Crassin
   * @return order : the order value
   * 
   */  
  public void setOrder(boolean order) {
    this.order = order;
  }  
  
  
  /**
   * Init the grid with appropriate titles list and columns
   * 
   * @author Jean-Yves Goubet
   * 
   */
  protected void manageGrid() {
    //
    this.resize(1, columnTitles.length);
    // declarations
    for (int i = 0; i < columnTitles.length; i++) {
      TitlePanel titlePanel = new TitlePanel(columnTitles[i], i, this, activeSort);
      this.setWidget(0, i, titlePanel);
      if (activeSort && sortColumnIndex == i) {
        titlePanel.sortIconDisplay(order);
      }
    }
    this.getRowFormatter().setStyleName(0, "titleBarStyle");
  }
  
  
  /**
   * Method to add a row data at the end of the grid.  
   * 
   * @author Mathieu Crassin
   * @param data : tab of rowLabel that represent the different columns.
   * 
   */  
  public void addData(Widget []data) {
    if (data.length <= this.columnTitles.length) {
      // add a row at the end of the grid
      int rowIndex = this.getRowCount();
      this.resizeRows(rowIndex + 1);
      // fill the row with data
      for (int i = 0; i < data.length; i++) {
        if (data[i] instanceof RowLabel) {
          addLabelHandler((RowLabel) data[i]);
        }
        this.setWidget(rowIndex, i, data[i]);
      }
      // add style to row
      String style = "";
      if ((rowIndex - 1) % 2 == 0) {
        style = "impairStyle";
      } else {
        style = "pairStyle";
      }
      this.getRowFormatter().setStylePrimaryName(rowIndex, style);
    }
  }
  
  
  /**
   * This method is used to add mouse over and mouse out handlers on actions panel
   * 
   * @author Jean-Yves Goubet
   * @param focusPanel : the panel to add handlers
   * 
   */
  protected void addActionsHandler(final IndexFocusPanel focusPanel) {
    focusPanel.addMouseOverHandler(new MouseOverHandler() {
      public void onMouseOver(MouseOverEvent cursorPosition) {
        changeRowStyle(focusPanel.getRowIndex(),WHICHSTYLE);
        MouseOutHandlerWithHandlerRegistration mouseOutHandlerWithHandlerRegistration = 
          new MouseOutHandlerWithHandlerRegistration(focusPanel, SpecialGrid.this);
        mouseOutHandlerWithHandlerRegistration.setHandlerRegistration(focusPanel.
            addMouseOutHandler(mouseOutHandlerWithHandlerRegistration));
      }
    });
  } 
  
  
  /**
   * This method is used to instantiate mouseout handler in mouseover handler for a label.
   * 
   * @author Jean-Yves Goubet
   * @param label : label to add handlers
   * 
   */
  private void addLabelHandler(final RowLabel label) {
    label.addMouseOverHandler(new MouseOverHandler() {
      public void onMouseOver(MouseOverEvent cursorPosition) {
        changeRowStyle(label.getRowIndex(),WHICHSTYLE);
        MouseOutHandlerWithHandlerRegistration mouseOutHandlerWithHandlerRegistration = 
          new MouseOutHandlerWithHandlerRegistration(label, SpecialGrid.this);
        mouseOutHandlerWithHandlerRegistration.setHandlerRegistration(label.
            addMouseOutHandler(mouseOutHandlerWithHandlerRegistration));
      }
    });
  } 
  
  
  /**
   * This method is used to format a java.util.Date in string to be displayed.
   * 
   * @author Jean-Yves Goubet
   * @param dateInDate : date in java.util.Date format
   * @return dateInString : date in our own string format
   * 
   */
  public static String dateFormatter(Date dateInDate) {
    String dateInString = null;
    if (dateInDate == null) {
      dateInString = "N/A";
    } else {
      DateTimeFormat  processDateFormat = DateTimeFormat.getFormat("MM/dd/yyyy 'at' HH:mm:ss");
      dateInString = processDateFormat.format(dateInDate);
    }
    return dateInString;
  }  
  
  
  /**
   * This method checks if the length of the label is not too wide for the grid and cuts the string and add title if yes.
   * 
   * @author Jean-Yves Goubet
   * @param toFormat : String to check
   * @param maxSize : Maximum length of the string
   * @param rowIndex : Index of the label's row
   * @return RowLabel : the label instantiated correctly
   * 
   */
  protected RowLabel formatLabel(final String toFormat, final int maxSize, final int rowIndex) {
    RowLabel formatedLabel;
    if (toFormat != null) {
      // check if length is too big
      if (toFormat.length() >= maxSize) {
        if (maxSize > exceededString.length()) {
          formatedLabel = new RowLabel(toFormat.substring(0,maxSize - exceededString.length()) + exceededString, rowIndex,false);
          formatedLabel.setTitle(toFormat);
        } else {
          formatedLabel = new RowLabel(toFormat.substring(0,maxSize), rowIndex,false);
          formatedLabel.setTitle(toFormat);
        }
      } else {
        formatedLabel = new RowLabel(toFormat,rowIndex,false);
      }
    } else {
      formatedLabel = new RowLabel("", rowIndex, false);
    }
    return formatedLabel;
  }  
  
  
  /**
   * Change the style of a row (in our grid).
   * 
   * @author Jean-Yves Goubet
   * @param rowIndex : index of the row to style.
   * @param whichStyle : string use to construct the style name.
   * 
   */
  public void changeRowStyle(int rowIndex, String whichStyle) {
    String style;
    if ((rowIndex - 1) % 2 == 0) {
      style = "impair" + whichStyle + "Style";
    } else {
      style = "pair" + whichStyle + "Style";
    }
    this.getRowFormatter().setStyleName(rowIndex, style);
  }  
  
  
  /**
   * We declare an empty methods because the child class had to redefine it.
   * 
   */
  public abstract void sortAndDisplayList();
  protected abstract void refreshCurrentList();
  
  
  /**
   * This function add an action (an icon in action column) for a specified row and handlers associated
   * This procedure is defined in child class and define the role of each action (by adding handler).
   * 
   * @author Mathieu Crassin
   * @param tab : contain the definition of each action. 
   * @param stateOfTheLine : the state of the line
   * @param columnIndex : the index of the row. 
   * 
   */
  protected void addActions(final ActionImage [] tab, String stateOfTheLine, int columnIndex) {
    // if tab is empty
    if (tab.length <= 0) {
      return;
    }
    
    // declarations
    final IndexFocusPanel focusPanel  = new IndexFocusPanel(tab[0].getRowIndex(),false);
    final HorizontalPanel actions = new HorizontalPanel();
    
    // instantiations
    focusPanel.add(actions);
    focusPanel.addFocusHandler(new FocusHandler() {
      public void onFocus(FocusEvent arg0) {
        focusPanel.setFocus(false);        
      }
    });
    addActionsHandler(focusPanel);  
    
    actions.setSize("100%", "23px");
    actions.setVerticalAlignment(HasVerticalAlignment.ALIGN_MIDDLE);
    actions.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);

    // add handler to each image
    addClickHandlers(tab, stateOfTheLine);
       
    // Add images to panel
    for (int i = 0; i < tab.length; i++) {
      actions.add(tab[i]);
    }
    
    // Add panel to grid cell
    this.setWidget(tab[0].getRowIndex(), columnIndex, focusPanel);
  }

  
  /** 
   * This function must be implemented in child class, it's used to define the handler 
   * associated with each image in actions column. 
   * 
   * @param tabImages : the images in actions column
   * @param state : the state of images
   * 
   */
  protected abstract void addClickHandlers(ActionImage[] tabImages, String state);
  
}
