/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client;


import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

import java.util.List;

import org.ow2.orchestra.console.GWTConsole.client.ListInstance.MInstance;
import org.ow2.orchestra.console.GWTConsole.client.ListInstanceActivity.MInstanceActivity;
import org.ow2.orchestra.console.GWTConsole.client.ListProcess.MProcess;
import org.ow2.orchestra.console.GWTConsole.client.ListProcessActivity.MProcessActivity;

/**
 * @author Mathieu Crassin
 *
 * Interface of the OrchestraManagement service
 */
@RemoteServiceRelativePath("OrchestraManagement")
public interface OrchestraManagementServices extends RemoteService {
  
  /**
   * Get the file named filename on the disk and deploy it to orchestra.
   * 
   * @author Mathieu Crassin
   * @param filename : filename of the file to deploy
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                them to the client side
   */
  void deployToOrchestra(final String filename) throws OrchestraManagementException;

  /**
   * Get the process list deploy on the orchestra server.  
   *
   * @author Jean-Yves Goubet
   * @return List<MProcess> : The  process list.
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  List<MProcess> getProcesses() throws OrchestraManagementException;
  MProcess getProcess(final String processUUID) throws OrchestraManagementException;
  
  
  /**
   * Undeploy a process from the orchestra server thanks to its UUID. 
   * 
   * @author Mathieu Crassin
   * @param processUUID : Information about the process to undeploy.
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void undeployFromOrchestra(final String processUUID) throws OrchestraManagementException;
  
  /**
   * Get the instances list on the orchestra server
   * 
   * @author Jean-Yves Goubet
   * @param processId : specify it to get the instances for this specific process. When null, all instances will be returned.
   * @return List<MInstance> : The list filled with instances
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  List<MInstance> getInstances(String processId) throws OrchestraManagementException;
  MInstance getInstance(String instanceId) throws OrchestraManagementException;
  
  /**
   * Get the activities of a given process
   * 
   * @author Jean-Yves Goubet
   * @param processId : it's the UUID of the process
   * @return List<MProcessActivity> : The list filled with process' activities
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  List<MProcessActivity> getProcessActivities(String processId) throws OrchestraManagementException;
  
  /**
   * Get the activities of a given instance
   * 
   * @author Jean-Yves Goubet
   * @param instanceId : it's the UUID of the instance
   * @return List<MInstanceActivity> : The list filled with instance' activities
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  List<MInstanceActivity> getInstanceActivities(String instanceId) throws OrchestraManagementException;
  MInstanceActivity getActivity(String activityId) throws OrchestraManagementException;
  
  /**
   * Suspend a running instance.
   * 
   * @author Jean-Yves Goubet
   * 
   * @param instanceId : the id of the instance to suspend
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void suspendInstance(String instanceId) throws OrchestraManagementException;
  
  /**
   * Resume a suspended instance.
   * 
   * @author Jean-Yves Goubet
   * 
   * @param instanceId : the id of the instance to resume
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void resumeInstance(String instanceId) throws OrchestraManagementException;
  
  /**
   * Exit an instance.
   * 
   * @author Jean-Yves Goubet
   * 
   * @param instanceId : the id of the instance to exit
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void exitInstance(String instanceId) throws OrchestraManagementException;
  
  /**
   * Remove an instance.
   * 
   * @author Jean-Yves Goubet
   * 
   * @param instanceId : the id of the instance to remove
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void removeInstance(String instanceId) throws OrchestraManagementException;
  
  /**
   * Remove a process.
   * 
   * @author Jean-Yves Goubet
   * 
   * @param instanceId : the id of the process to remove
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void removeProcess(String processId) throws OrchestraManagementException;
  
  /**
   * Log out the user
   * 
   * @author Jean-Yves Goubet
   * 
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void sessionLogout() throws OrchestraManagementException;
  
  /**
   * Get the username of the user
   * 
   * @author Jean-Yves Goubet
   * @return String : the username
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  String getSessionUserName() throws OrchestraManagementException;
  
  /**
   * Method to get the date of the session's beginning
   * 
   * @author Jean-Yves Goubet
   * @return long : the date
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  long getSessionCreationDate() throws OrchestraManagementException;
}
