/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client;


import com.google.gwt.user.client.rpc.AsyncCallback;

import java.util.List;

import org.ow2.orchestra.console.GWTConsole.client.ListInstance.MInstance;
import org.ow2.orchestra.console.GWTConsole.client.ListInstanceActivity.MInstanceActivity;
import org.ow2.orchestra.console.GWTConsole.client.ListProcess.MProcess;
import org.ow2.orchestra.console.GWTConsole.client.ListProcessActivity.MProcessActivity;


/**
 * Interface of the OrchestraManagement service
 * 
 * @author Mathieu Crassin
 *
 */
public interface OrchestraManagementServicesAsync {
  
  /**
   * Get the file named filename on the disk and deploy it to orchestra.
   * 
   * @author Mathieu Crassin
   * @param filename : filename of the file to deploy
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                them to the client side
   */
  void deployToOrchestra(final String filename, AsyncCallback<Void> callback);
  
  /**
   * Get the process list deploy on the orchestra server.  
   *
   * @author Jean-Yves Goubet
   * @return List<MProcess> : The  process list.
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void getProcesses(AsyncCallback<List<MProcess>> callback);
  void getProcess(final String processUUID, AsyncCallback<MProcess> callback);
  
  /**
   * Undeploy a process from the orchestra server thanks to its UUID. 
   * 
   * @author Mathieu Crassin
   * @param processUUID : Information about the process to undeploy.
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void undeployFromOrchestra(final String processUUID, AsyncCallback<Void> callback);

  /**
   * Get the instances list on the orchestra server
   * 
   * @author Jean-Yves Goubet
   * @param processId : specify it to get the instances for this specific process. When null, all instances will be returned.
   * @return List<MInstance> : The list filled with instances
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void getInstances(String processId, AsyncCallback<List<MInstance>> callback);
  void getInstance(String instanceId, AsyncCallback<MInstance> callback);
  
  /**
   * Get the activities of a given process
   * 
   * @author Jean-Yves Goubet
   * @param processId : it's the UUID of the process
   * @return List<MProcessActivity> : The list filled with process' activities
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void getProcessActivities(String processId, AsyncCallback<List<MProcessActivity>> callback);
  
  /**
   * Get the activities of a given instance
   * 
   * @author Jean-Yves Goubet
   * @param instanceId : it's the UUID of the instance
   * @return List<MInstanceActivity> : The list filled with instance' activities
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void getInstanceActivities(String instanceId, AsyncCallback<List<MInstanceActivity>> callback);
  void getActivity(String activityId, AsyncCallback<MInstanceActivity> callback);
  
  /**
   * Suspend a running instance.
   * 
   * @author Jean-Yves Goubet
   * 
   * @param instanceId : the id of the instance to suspend
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void suspendInstance(String instanceId, AsyncCallback<Void> callback);
  
  /**
   * Resume a suspended instance.
   * 
   * @author Jean-Yves Goubet
   * 
   * @param instanceId : the id of the instance to resume
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void resumeInstance(String instanceId, AsyncCallback<Void> callback);
  
  /**
   * Exit an instance.
   * 
   * @author Jean-Yves Goubet
   * 
   * @param instanceId : the id of the instance to exit
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void exitInstance(String instanceId, AsyncCallback<Void> callback);
  
  /**
   * Remove an instance.
   * 
   * @author Jean-Yves Goubet
   * 
   * @param instanceId : the id of the instance to remove
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void removeInstance(String instanceId, AsyncCallback<Void> callback);
  
  /**
   * Remove a process.
   * 
   * @author Jean-Yves Goubet
   * 
   * @param instanceId : the id of the process to remove
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void removeProcess(String processId, AsyncCallback<Void> callback);
  
  /**
   * Log out the user
   * 
   * @author Jean-Yves Goubet
   * 
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void sessionLogout(AsyncCallback<Void> callback);
  
  /**
   * Get the username of the user
   * 
   * @author Jean-Yves Goubet
   * @return String : the username
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void getSessionUserName(AsyncCallback<String> callback);
  
  /**
   * Method to get the date of the session's beginning
   * 
   * @author Jean-Yves Goubet
   * @return long : the date
   * @throws OrchestraManagementException : to get some orchestra exception and transmit
   *                                        them to the client side
   */
  void getSessionCreationDate(AsyncCallback<Long> callback);

}
