/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.Popup;

import org.ow2.orchestra.console.GWTConsole.client.Commons.PresenterIF;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.logical.shared.CloseEvent;
import com.google.gwt.event.logical.shared.CloseHandler;
import com.google.gwt.event.logical.shared.OpenEvent;
import com.google.gwt.event.logical.shared.OpenHandler;
import com.google.gwt.user.client.ui.DisclosurePanel;

/**
 * @author Mathieu Crassin
 *
 * Error popup presenter class.
 */
public class PPopup extends PresenterIF {

  /**
   * @author Mathieu Crassin
   *
   * Interface that represent the view of the popup.
   */
  protected VPopupIF view;
  /**
   * Type of the popup.
   * 
   * @author Mathieu Crassin
   */
  public enum PopupType {
    ErrorPopup,
    InfoPopup
  }  
  
  /**
   * Presenter constructor of the popup.
   * We have to:
   *  - create the view
   *  - bind the view
   *  - add widgets listeners
   *  - and others things
   * 
   * @author Mathieu Crassin
   * @param maininfo : main information of the popup.
   * @param details : details information of the popup (if details is null, no display)
   */
  public PPopup(final PopupType type, final String maininfo, final String details) {
    //
    final VPopup ep = new VPopup(maininfo, details);
    //
    this.bindingView(ep);
    //
    this.setDebugID();
    //
    this.addListeners();
    // display or not the disclosure panel and text area for the details information
    this.manageDetails(details);
    // select info or error type for the popup
    this.definePopupType(type);
  }
  public PPopup(final PopupType type, final String maininfo) {
    this(type, maininfo, null);
  }  
  
  /**
   * Define an id for each widget. 
   * This Id is used by selenium for the integration phase.
   * 
   * @author Mathieu Crassin
   *
   */  
  protected void setDebugID() {
    this.view.getDialogBox().ensureDebugId("popupId");

    this.view.getInfoLabel().ensureDebugId("popupInfoLabelId");
    this.view.getErrorImage().ensureDebugId("popupErrorImageId");
    this.view.getInfoImage().ensureDebugId("popupInfoImageId");
    this.view.getDetailsTextarea().ensureDebugId("popupDetailsTextareaId");
    this.view.getCloseButton().ensureDebugId("popupCloseButtonId");
  }
  
  /**
   * Define the type of the popup. This will change the left image, the title and its color.  
   * 
   * @author Mathieu Crassin
   * @param type
   */
  private void definePopupType(PopupType type) {
    
    if (type == PopupType.ErrorPopup) {
      //
      this.view.getErrorImage().setVisible(true);
      this.view.getInfoImage().setVisible(false);
      //
      this.view.getDialogBox().setText("Error !");
      //
      this.view.getDialogBox().removeStyleName("infoPopup-DialogBox");
      this.view.getDialogBox().addStyleName("errorPopup-DialogBox");
      
    } else if (type == PopupType.InfoPopup) {
      //
      this.view.getErrorImage().setVisible(false);
      this.view.getInfoImage().setVisible(true);
      //
      this.view.getDialogBox().setText("Information !");
      //
      this.view.getDialogBox().removeStyleName("errorPopup-DialogBox");
      this.view.getDialogBox().addStyleName("infoPopup-DialogBox");
    }
    
  }  

  /**
   * @author Mathieu Crassin
   * @param details : details information of the popup.
   *
   * Display or not the disclosure panel that hold the details information.
   */
  private void manageDetails(final String details) {
    if (details == null || details.equals("")) {
      this.view.getDisclosurePanel().setVisible(false);
    } else {
      this.view.getDisclosurePanel().setVisible(true);
    }
  }

  /**
   * @author Mathieu Crassin
   * @param ep : interface of the view.
   *
   * Binding the view.
   */
  protected void bindingView(final Object ep) {
    this.view = (VPopupIF) ep;
  }

  /**
   * @author Mathieu Crassin
   *
   * Add the listeners of several widgets.
   */
  protected void addListeners() {
    // close button
    this.view.getCloseButton().addClickHandler(new ClickHandler() {
      public void onClick(final ClickEvent arg0) {
        PPopup.this.view.getDialogBox().hide();
      }
    });
    // disclosure panel
    this.view.getDisclosurePanel().addOpenHandler(new OpenHandler<DisclosurePanel>() {
      public void onOpen(final OpenEvent<DisclosurePanel> arg0) {
        PPopup.this.view.getDisclosurePanel().getHeaderTextAccessor().setText("Less Details");
      }
    });
    this.view.getDisclosurePanel().addCloseHandler(new CloseHandler<DisclosurePanel>() {
      public void onClose(final CloseEvent<DisclosurePanel> arg0) {
        PPopup.this.view.getDisclosurePanel().getHeaderTextAccessor().setText("More Details");
      }
    });
  }

  /**
   * @author Mathieu Crassin
   * @param maininfo : main information of the popup.
   * @param details : details information of the popup.
   *
   * Static function that allow class users to start
   * an error popup just by calling it.
   */
  public static final void launch(final PopupType type, final String maininfo, final String details) {
    new PPopup(type, maininfo, details);
  }

  
  public void destroy() {
    // no operations on destroy
    
  }
}
