/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.server;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.logging.Level;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.io.FilenameUtils;
import org.ow2.orchestra.util.Misc;

/**
 *
 * @author Mathieu Crassin
 *
 */
@SuppressWarnings("serial")
public class FileUploadServlet extends HttpServlet {

  /**
   * @author Mathieu Crassin
   *
   * Work directory on the upload servlet.
   */
  private String rootDirectory;

  public static final String ENDLINE_STR = ";";
  public static final String MESSAGE_ERROR_STR = "Error: ";
  public static final String DETAILS_ERROR_STR = "Details: ";
  
  
  /**
   * @author mathieu
   * @throws ServletException Exception about http Servlet
   *
   * Constructor of the class.
   */
  public final void init() throws ServletException {
    //
    this.rootDirectory = System.getProperty("java.io.tmpdir");
    //
    checkDirectory();
  }

  
  public final void setRootDirectory(String rootDirectory) {
    this.rootDirectory = rootDirectory;
  }
  
  
  /**
   * @author mathieu
   *
   * This function test if the tmp directory exists, else it created.
   */
  protected final boolean checkDirectory() {
    File tmpdir = new File(rootDirectory);
    Misc.log(Level.INFO,"root directory = " + tmpdir.getAbsolutePath());
    if (!tmpdir.isDirectory()) { // the directory doesn't exist
      Misc.log(Level.WARNING,"Temp directory doesn't exists !");
      return false;
    } else {
      rootDirectory = tmpdir.getAbsolutePath() + File.separator;
System.out.println("rootDirectory = " + rootDirectory);
      return true;
    }
  }

  /**
   * @author mathieu
   * @param request the request
   * @param response the response
   * @throws ServletException exception about the http servlet
   * @throws IOException exception about the read and write operations
   *
   * This method is called when a post request comes on the servlet
   * (at /DeployBar).
   * The request must be multipart and contain one file.
   * Then the file is copied in the rootDirectory.
   * The complete path of the file is returned.
   */
  protected final void doPost(final HttpServletRequest request, 
                              final HttpServletResponse response) throws ServletException, IOException {
    Misc.log(Level.INFO,"doPost method start");
    FileItem item = doPostParse(request, response);
    if (item != null) {
      Misc.log(Level.INFO,"doPost : Request parsed");
      
      if (doPostCopy(item, response)) {
        Misc.log(Level.INFO,"doPost : item copied");
      } else {
        Misc.log(Level.INFO,"doPost : error in copy !");
      }      
      
    } else {
      Misc.log(Level.INFO,"doPost : Parse error !");
    } 
    Misc.log(Level.INFO,"End doPost method");
  }

  /**
   * @author Mathieu Crassin
   * @param request receive from the client
   * @param response ressponse send to client
   * @return FileItem of the request after parsing it
   *
   * Parse request and return the file.
   */
  protected final FileItem doPostParse(final HttpServletRequest request, final HttpServletResponse response) {

    boolean isMultipart = ServletFileUpload.isMultipartContent(request);
    if (isMultipart) {
      Misc.log(Level.INFO,"doPostParse: multipart request");
      FileItemFactory factory = new DiskFileItemFactory();
      ServletFileUpload upload = new ServletFileUpload(factory);
      upload.setHeaderEncoding("UTF-8");
      try {
        List< ? > items = upload.parseRequest(request);
        Misc.log(Level.INFO,"doPostParse: nb items=" + items.size());
        if (items.size() == 1) {
          FileItem item = (FileItem) items.get(0);
          Misc.log(Level.INFO,"doPostParse: item=" + item.getName());
          if (item != null) {
            return item;
          } else {
            signalError(response,
                "Error while uploading file !", "FileUploadServlet - doPostParse : Error while processing item !", 
                "FileUploadServlet - doPostParse : Error while processing item !");         
            return null;
          }
        } else {
          signalError(response, "Error while uploading file !",
              "FileUploadServlet - doPostParse : Error, only one item is expected in a post request !",
              "FileUploadServlet - doPostParse : Error, only one item is expected in a post request !");
          return null;
        }
      } catch (FileUploadException e) {
        signalError(response, "Error while uploading file !",
            "FileUploadServlet - doPostParse : Exception in fileupload processing !" + "\n" + Misc.getStackTraceFrom(e),
            "FileUploadServlet - doPostParse : Exception in fileupload processing !" + "\n" + Misc.getStackTraceFrom(e));
        return null;
      }
    } else {
      return null;
    }
  }

  /**
   * @author Mathieu Crassin
   * @param item parsing from the doPostParse method
   * @param response request use to return some message to the client
   * @return boolean whether or not the function succeed
   *
   * Use the item class to get the file, and copy it in temp directory.
   */
  protected final boolean doPostCopy(final FileItem item,
                                     final HttpServletResponse response) {

    String filename = item.getName();
    if (filename != null && !filename.equals("")) {
      try {
        // check if file exists
        if (item.getSize() == 0) {
          // write return message
          signalError(response,"Error while uploading file !",
              "FileUploadServlet - doPostCopy : file " + rootDirectory + filename + " doesn't exist !",
              "doPostCopy: file " + rootDirectory + filename + " doesn't exist !");
          return false; // Error
        }
        // extract only the name from the file path
System.out.println("filename = " + filename);        
        filename = FilenameUtils.getName(filename);
        File uploadedFile = File.createTempFile("tmp", ".bar", new File(rootDirectory)); 
        // write return message
        String returnMsg = rootDirectory + uploadedFile.getName();
System.out.println("returnMsg = " + returnMsg);        
        response.getWriter().write(returnMsg);      
        // write the file in temp directory
        item.write(uploadedFile);
        Misc.log(Level.INFO,"doPostCopy: item copied");
        response.getWriter().close();
      } catch (Exception e) {
        Misc.log(Level.SEVERE,"doPostCopy: Exception happened during response processing !" + e);
        return false; // Error
      }
      return true; // OK
    } else {    
      signalError(response,
          "Error while uploading file !", "FileUploadServlet - doPostCopy : Incorrect filename !",
          "FileUploadServlet - doPostCopy: Incorrect filename !");
      return false; // Error
    }
  }
  
  /**
   * @author Mathieu Crassin
   * @param response
   * @param clientmessage
   * @param clientdetails
   * @param logmessage
   * 
   * Function that write in response stream the error with the correct syntax and
   * log the error with standard system. 
   */
  private void signalError(final HttpServletResponse response, 
                           final String clientmessage, final String clientdetails, 
                           final String logmessage) {
    // send error to client
    try {
      response.getWriter().write(FileUploadServlet.MESSAGE_ERROR_STR + clientmessage + FileUploadServlet.ENDLINE_STR);
      response.getWriter().write(FileUploadServlet.DETAILS_ERROR_STR + clientdetails + FileUploadServlet.ENDLINE_STR);
    } catch (IOException e) {
      Misc.log(Level.SEVERE,"FileUploadServlet - doPost : Error while sending response !");
    }
    // log the information
    Misc.log(Level.SEVERE,logmessage);   
  }
}
